from redirect_urls.utils import (
    gone,
    header_redirector,
    is_firefox_redirector,
    no_redirect,
    platform_redirector,
    redirect,
    ua_redirector,
)


__version__ = '0.1'
default_app_config = 'redirect_urls.apps.RedirectsConfig'
